@@ -1728,7 +1728,7 @@ static void updateGrid (struct state *st)
                 s[j] = (random() % 10) + '0';
               s[j++] = '?';
               s[j++] = ':';
-              s[j++] = 0;
+              s[j] = 0;
             }
 
           /* change one digit in this barcode */
@@ -1911,6 +1911,7 @@ barcode_free (Display *dpy, Window window, void *closure)
 static const char *barcode_defaults [] = {
     ".background:	black",
     ".foreground:	green",
+    "*fpsSolid: 	true",
     "*delay:		10000",
     "*mode:		scroll",
     0
