@@ -1,4 +1,4 @@
-/* fps, Copyright (c) 2001-2008 Jamie Zawinski <jwz@jwz.org>
+/* fps, Copyright (c) 2001-2011 Jamie Zawinski <jwz@jwz.org>
  * Draw a frames-per-second display (Xlib and OpenGL).
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -23,7 +23,6 @@ fps_init (Display *dpy, Window window)
   fps_state *st;
   const char *font;
   XFontStruct *f;
-  int first, last;
 
   if (! get_boolean_resource (dpy, "doFPS", "DoFPS"))
     return 0;
@@ -40,9 +39,6 @@ fps_init (Display *dpy, Window window)
   f = XLoadQueryFont (dpy, font);
   if (!f) f = XLoadQueryFont (dpy, "fixed");
 
-  first = f->min_char_or_byte2;
-  last = f->max_char_or_byte2;
-
   {
     XWindowAttributes xgwa;
     XGCValues gcv;
@@ -85,7 +81,7 @@ fps_slept (fps_state *st, unsigned long usecs)
 
 
 double
-fps_compute (fps_state *st, unsigned long polys)
+fps_compute (fps_state *st, unsigned long polys, double depth)
 {
   if (! st) return 0;  /* too early? */
 
@@ -153,6 +149,18 @@ fps_compute (fps_state *st, unsigned long polys)
           else
             sprintf (st->string + strlen(st->string), "%lu%s ", polys, s);
         }
+
+      if (depth >= 0.0)
+        {
+          int L = strlen (st->string);
+          char *s = st->string + L;
+          strcat (s, "\nDepth: ");
+          sprintf (s + strlen(s), "%.1f", depth);
+          L = strlen (s);
+          /* Remove trailing ".0" in case depth is not a fraction. */
+          if (s[L-2] == '.' && s[L-1] == '0')
+            s[L-2] = 0;
+        }
     }
 
   return st->last_fps;
@@ -219,7 +227,6 @@ fps_draw (fps_state *st)
   if (st->clear_p)
     {
       int w, h;
-      int lh = st->font->ascent + st->font->descent;
       w = string_width (st->font, string, &h);
       XFillRectangle (st->dpy, st->window, st->erase_gc,
                       x - st->font->descent,
