@@ -1,6 +1,11 @@
 /*
+ * models for the xss chess screensavers
+ * hacked from:
+ *
  * glChess - A 3D chess interface
  *
+ * Copyright (C) 2006  John-Paul Gignac <jjgignac@users.sf.net>
+ *
  * Copyright (C) 2002  Robert  Ancell <bob27@users.sourceforge.net>
  *                     Michael Duelli <duelli@users.sourceforge.net>
  *
@@ -19,13 +24,21 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-void revolve_line(double *, double *, double, int);
-void gen_model_lists(void);
-void draw_pawn(void);
-void draw_rook(void);
-void draw_rook_old(void);
-void draw_knight(void);
-void draw_bishop(void);
-void draw_queen(void);
-void draw_king(void);
+/* ugggggggly */
+#define PIECES    7
+#define NONE      0
+#define KING      1
+#define QUEEN     2
+#define BISHOP    3 
+#define KNIGHT    4 
+#define ROOK      5
+#define PAWN      6 
+#define BKING     8
+#define BQUEEN    9
+#define BBISHOP  10 
+#define BKNIGHT  11
+#define BROOK    12
+#define BPAWN    13 
+
+extern void gen_model_lists( int classic, int poly_count[PIECES]);
 
