@@ -13,18 +13,7 @@
  * Written by Linas Vepstas, March 1995
  */
 
-/* required include files */
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#include <GL/gl.h>
-/*#include <GL/glut.h>*/
-#ifdef HAVE_GLE3
-#include <GL/gle.h>
-#else
-#include <GL/tube.h>
-#endif
+#include "extrusion.h"
 
 /* controls shape of object */
 extern float lastx;
