@@ -1,4 +1,4 @@
-/* pinion, Copyright (c) 2004-2008 Jamie Zawinski <jwz@jwz.org>
+/* pinion, Copyright (c) 2004-2011 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -12,9 +12,9 @@
 #define DEFAULTS        "*delay:        15000              \n" \
                         "*showFPS:      False              \n" \
                         "*wireframe:    False              \n" \
-                        "*titleFont:  -*-times-bold-r-normal-*-180-*\n" \
-                        "*titleFont2: -*-times-bold-r-normal-*-120-*\n" \
-                        "*titleFont3: -*-times-bold-r-normal-*-80-*\n"  \
+                        "*titleFont:  -*-helvetica-medium-r-normal-*-180-*\n" \
+                        "*titleFont2: -*-helvetica-medium-r-normal-*-120-*\n" \
+                        "*titleFont3: -*-helvetica-medium-r-normal-*-80-*\n"  \
 
 # define refresh_pinion 0
 # define release_pinion 0
@@ -165,7 +165,7 @@ new_label (ModeInfo *mi)
       print_gl_string (mi->dpy, f, fl,
                        mi->xgwa.width, mi->xgwa.height,
                        10, mi->xgwa.height - 10,
-                       label);
+                       label, False);
     }
   glEndList ();
 }
@@ -883,6 +883,7 @@ push_gear (ModeInfo *mi)
 
   if (g->coax_p)
     {
+      if (!parent) abort();
       if (g->x != parent->x) abort();
       if (g->y != parent->y) abort();
       if (g->z == parent->z) abort();
@@ -1340,8 +1341,6 @@ init_pinion (ModeInfo *mi)
       fprintf(stderr, "%s: out of memory\n", progname);
       exit(1);
     }
-
-    pp = &pps[MI_SCREEN(mi)];
   }
 
   pp = &pps[MI_SCREEN(mi)];
@@ -1350,6 +1349,7 @@ init_pinion (ModeInfo *mi)
 
   load_fonts (mi);
   reshape_pinion (mi, MI_WIDTH(mi), MI_HEIGHT(mi));
+  clear_gl_error(); /* WTF? sometimes "invalid op" from glViewport! */
 
   pp->title_list  = glGenLists (1);
 
