@@ -1,4 +1,4 @@
-/* topblock, Copyright (c) 2006-2008
+/* topblock, Copyright (c) 2006-2009
  *  rednuht <topblock.xscreensaver@jumpstation.co.uk>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -189,10 +189,7 @@ init_topBlock (ModeInfo *mi)
   if (!tbs) {
     tbs = (topBlockSTATE *)
       calloc (MI_NUM_SCREENS(mi), sizeof (topBlockSTATE));
-    if (!tbs) {
-      fprintf(stderr, "%s: out of memory\n", progname);
-      exit(1);
-    }
+    if (!tbs) abort();
   }
 
   tb = &tbs[MI_SCREEN(mi)];
@@ -231,14 +228,7 @@ init_topBlock (ModeInfo *mi)
 	dropSpeed = 80/dropSpeed;
 	dropSpeed = (blockHeight/dropSpeed); 
 
-  tb->glx_context = init_GL(mi);
-
   reshape_topBlock (mi, MI_WIDTH(mi), MI_HEIGHT(mi));
-  if (0==1) {
-      glClearColor(1.0f, 1.0f, 1.0f, 0.5f);
-  } else {
-      glClearColor(0.0f, 0.0f, 0.0f, 0.5f);
-  }
   glClearDepth(1.0f);
   if (!wire) {
     GLfloat pos[4] = {10.0, 10.0, 1.0, 0.0};
@@ -578,16 +568,18 @@ static void generateNewBlock(ModeInfo *mi)
 		tb->numFallingBlocks++;
 		llTail = tb->blockNodeRoot; 
 		if (llTail == NULL) {
-			if ((llCurrent = ((NODE*) malloc(sizeof(NODE)))) == NULL) {	fprintf(stderr, "%s: out of memory.\n", progname); }
-			llTail = llCurrent;
-			tb->blockNodeRoot = llCurrent; 
+                  llCurrent = ((NODE*) malloc(sizeof(NODE)));
+                  if (!llCurrent) abort();
+                  llTail = llCurrent;
+                  tb->blockNodeRoot = llCurrent; 
 		} else {
 			if (tb->numFallingBlocks>=maxFalling) {
 				/* recycle */
 				llCurrent=llTail->next;
 				tb->blockNodeRoot=llCurrent->next;
 			} else {
-				if ((llCurrent = ((NODE*) malloc(sizeof(NODE)))) == NULL) {	fprintf(stderr, "%s: out of memory..\n", progname); }
+                          llCurrent = ((NODE*) malloc(sizeof(NODE)));
+                          if (!llCurrent) abort();
 			}
 			while (llTail->next != NULL) { llTail = llTail->next; } /* find last item in list */
 		}
