@@ -243,7 +243,6 @@ static void
 setup_display (struct state *st)
 {
     XWindowAttributes xgwa;
-    Colormap cmap;
 
     int cell_size = get_integer_resource (st->dpy, "size", "Integer");
     int osize, alloc_size, oalloc;
@@ -398,8 +397,6 @@ setup_display (struct state *st)
     st->mindeathspeed *= st->diaglim;
     st->maxdeathspeed *= st->diaglim;
 
-    cmap = xgwa.colormap;
-    
     st->windowWidth = xgwa.width;
     st->windowHeight = xgwa.height;
     
@@ -718,6 +715,7 @@ petri_free (Display *dpy, Window window, void *closure)
 static const char *petri_defaults [] = {
   ".background:		black",
   ".foreground:		white",
+  "*fpsSolid:		true",
   "*delay:		10000",
   "*count:		20",
   "*size:		2",
