@@ -90,6 +90,8 @@ struct state {
   int draw_count;
 
   int iterations, delay, rate, box, oily, stir, fluidity;
+  int duration;
+  time_t start_time;
 
   void (*draw_transparent) (struct state *st, short *src);
 
@@ -529,6 +531,7 @@ setup_X(struct state *st)
 
     st->img_loader = load_image_async_simple (0, xgwa.screen, st->window,
                                               st->window, 0, 0);
+    st->start_time = time ((time_t) 0);
   } else {
     XGCValues gcv;
 
@@ -700,6 +703,7 @@ sinc(struct state *st, double x)
   int i;
   i = (int)(x * TABLE + 0.5);
   if (i >= TABLE) i = (TABLE-1) - (i-(TABLE-1));
+  if (i < 0) return 0.;
   return st->cos_tab[i];
 #elif 0
   return cos(x * M_PI/2);
@@ -715,19 +719,21 @@ sinc(struct state *st, double x)
 static void
 add_circle_drop(struct state *st, int x, int y, int radius, int dheight)
 {
-  int i, r2, cx, cy;
+  int r, r2, cx, cy;
   short *buf = (random()&1) ? st->bufferA : st->bufferB;
 
-  i = y * st->width + x;
   r2 = radius * radius;
 
+
   for (cy = -radius; cy <= radius; cy++)
     for (cx = -radius; cx <= radius; cx++) {
-      int r = cx*cx + cy*cy;
-      if (r <= r2) {
-        buf[i + cx + cy*st->width] =
-          (short)(dheight * sinc(st, sqrt(r)/radius));
-      }
+      int xx = x+cx;
+      int yy = y+cy;
+      if (xx < 0 || yy < 0 || xx >= st->width || yy >= st->height) {break;}
+      r = cx*cx + cy*cy;
+      if (r > r2) break;
+      buf[xx + yy*st->width] =
+        (short)(dheight * sinc(st, (radius > 0) ? sqrt(r)/radius : 0));
     }
 }
 
@@ -951,6 +957,7 @@ ripples_init (Display *disp, Window win)
   st->window = win;
   st->iterations = 0;
   st->delay = get_integer_resource(disp, "delay", "Integer");
+  st->duration = get_integer_resource (st->dpy, "duration", "Seconds");
   st->rate = get_integer_resource(disp, "rate", "Integer");
   st->box = get_integer_resource(disp, "box", "Integer");
   st->oily = get_boolean_resource(disp, "oily", "Boolean");
@@ -963,6 +970,8 @@ ripples_init (Display *disp, Window win)
 #endif /* HAVE_XSHM_EXTENSION */
   st->light = get_integer_resource(disp, "light", "Integer");
 
+  if (st->delay < 0) st->delay = 0;
+  if (st->duration < 1) st->duration = 1;
   if (st->fluidity <= 1) st->fluidity = 1;
   if (st->fluidity > 16) st->fluidity = 16; /* 16 = sizeof(short) */
   if (st->light < 0) st->light = 0;
@@ -1021,6 +1030,7 @@ ripples_draw (Display *dpy, Window window, void *closure)
       if (! st->img_loader) {  /* just finished */
         XWindowAttributes xgwa;
         XGetWindowAttributes(st->dpy, st->window, &xgwa);
+        st->start_time = time ((time_t) 0);
         st->orig_map = XGetImage (st->dpy, st->window, 0, 0, 
                                   xgwa.width, xgwa.height,
                                   ~0L, ZPixmap);
@@ -1029,6 +1039,16 @@ ripples_draw (Display *dpy, Window window, void *closure)
       return st->delay;
     }
 
+    if (!st->img_loader &&
+        st->start_time + st->duration < time ((time_t) 0)) {
+      XWindowAttributes xgwa;
+      XGetWindowAttributes(st->dpy, st->window, &xgwa);
+      st->img_loader = load_image_async_simple (0, xgwa.screen, st->window,
+                                                st->window, 0, 0);
+      st->start_time = time ((time_t) 0);
+      return st->delay;
+    }
+
     if (st->rate > 0 && (st->iterations % st->rate) == 0)
       add_drop(st, ripple_blob, -SPLASH);
     if (st->stir)
@@ -1071,13 +1091,14 @@ static const char *ripples_defaults[] =
   "*colors:		200",
   "*dontClearRoot:	True",
   "*delay:		50000",
+  "*duration:		120",
   "*rate:	 	5",
   "*box:	 	0",
-  "*water: 		False",
+  "*water: 		True",
   "*oily: 		False",
   "*stir: 		False",
   "*fluidity: 		6",
-  "*light: 		0",
+  "*light: 		4",
   "*grayscale: 		False",
 #ifdef HAVE_XSHM_EXTENSION
   "*useSHM: True",
@@ -1092,6 +1113,7 @@ static XrmOptionDescRec ripples_options[] =
   { "-colors",	".colors",	XrmoptionSepArg, 0},
   { "-colours",	".colors",	XrmoptionSepArg, 0},
   {"-delay",	".delay",	XrmoptionSepArg, 0},
+  {"-duration",	".duration",	XrmoptionSepArg, 0 },
   {"-rate",	".rate",	XrmoptionSepArg, 0},
   {"-box",	".box",		XrmoptionSepArg, 0},
   {"-water",	".water",	XrmoptionNoArg, "True"},
