@@ -30,7 +30,9 @@ static const char sccsid[] = "@(#)slip.c	5.00 2000/11/01 xlockmore";
 # define DEFAULTS	"*delay: 50000 \n" \
 					"*count: 35 \n" \
 					"*cycles: 50 \n" \
-					"*ncolors: 200 \n"
+					"*ncolors: 200 \n" \
+					"*fpsSolid:	true \n"
+
 # define refresh_slip 0
 # define slip_handle_event 0
 # include "xlockmore.h"		/* in xscreensaver distribution */
