@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 4 -*-
  * swirl --- swirly color-cycling patterns.
  */
-#if !defined( lint ) && !defined( SABER )
+#if 0
 static const char sccsid[] = "@(#)swirl.c	4.00 97/01/01 xlockmore";
 #endif
 
@@ -30,18 +30,17 @@ static const char sccsid[] = "@(#)swirl.c	4.00 97/01/01 xlockmore";
  */
 
 #ifdef STANDALONE
-# define PROGCLASS					"Swirl"
-# define HACK_INIT					init_swirl
-# define HACK_DRAW					draw_swirl
-# define swirl_opts					xlockmore_opts
 # define DEFAULTS	"*count:		5       \n"			\
 					"*delay:		10000   \n"			\
 					"*ncolors:		200     \n"			\
-					"*useSHM:		True    \n"
+					"*useSHM:		True    \n" \
+					"*fpsSolid:		true    \n" \
+
 # define SMOOTH_COLORS
 # define WRITABLE_COLORS
+# define reshape_swirl 0
+# define swirl_handle_event 0
 # include "xlockmore.h"				/* from the xscreensaver distribution */
-# include <X11/Xutil.h>
 # ifdef HAVE_XSHM_EXTENSION
 #  include "xshm.h"
 # endif /* HAVE_XSHM_EXTENSION */
@@ -50,7 +49,7 @@ static const char sccsid[] = "@(#)swirl.c	4.00 97/01/01 xlockmore";
 # undef HAVE_XSHM_EXTENSION
 #endif /* !STANDALONE */
 
-ModeSpecOpt swirl_opts = {
+ENTRYPOINT ModeSpecOpt swirl_opts = {
   0, NULL, 0, NULL, NULL };
 
 #include <time.h>
@@ -224,9 +223,7 @@ initialise_swirl(ModeInfo * mi, SWIRL_P swirl)
 #endif /* !STANDALONE */
 
 
-#ifdef STANDALONE
-# define MI_COLORMAP MI_WIN_COLORMAP
-#else /* !STANDALONE */
+#ifndef STANDALONE
 	swirl->fg = MI_FG_COLOR(mi);
 	swirl->bg = MI_BG_COLOR(mi);
 	swirl->fgcol.pixel = swirl->fg;
@@ -1248,7 +1245,7 @@ next_point(SWIRL_P swirl)
  *
  * -      win is the window to draw in
  */
-void
+ENTRYPOINT void
 init_swirl(ModeInfo * mi)
 {
 	Display    *display = MI_DISPLAY(mi);
@@ -1257,19 +1254,14 @@ init_swirl(ModeInfo * mi)
 
 	/* does the swirls array exist? */
 	if (swirls == NULL) {
-		int         i;
-
 		/* allocate an array, one entry for each screen */
-		swirls = (SWIRL_P) calloc(ScreenCount(display), sizeof (SWIRL));
-
-		/* initialise them all */
-		for (i = 0; i < ScreenCount(display); i++)
-			initialise_swirl(mi, &swirls[i]);
+		swirls = (SWIRL_P) calloc(MI_NUM_SCREENS(mi), sizeof (SWIRL));
 	}
 	/* get a pointer to this swirl */
 	swirl = &(swirls[MI_SCREEN(mi)]);
-
-	/* get window parameters */
+        initialise_swirl(mi, swirl);
+                
+        /* get window parameters */
 	swirl->win = window;
 	swirl->width = MI_WIN_WIDTH(mi);
 	swirl->height = MI_WIN_HEIGHT(mi);
@@ -1350,7 +1342,7 @@ init_swirl(ModeInfo * mi)
  *
  * -      win is the window to draw in
  */
-void
+ENTRYPOINT void
 draw_swirl(ModeInfo * mi)
 {
 	SWIRL_P     swirl = &(swirls[MI_SCREEN(mi)]);
@@ -1443,8 +1435,8 @@ draw_swirl(ModeInfo * mi)
 
 /****************************************************************/
 
-void
-release_swirl(ModeInfo * mi)
+ENTRYPOINT void
+release_swirl (ModeInfo * mi)
 {
 	/* does the swirls array exist? */
 	if (swirls != NULL) {
@@ -1454,10 +1446,14 @@ release_swirl(ModeInfo * mi)
 		for (i = 0; i < MI_NUM_SCREENS(mi); i++) {
 			SWIRL_P     swirl = &(swirls[i]);
 
+#ifndef STANDALONE
 			if (swirl->cmap != (Colormap) NULL)
 				XFreeColormap(MI_DISPLAY(mi), swirl->cmap);
+#endif /* STANDALONE */
+#ifndef STANDALONE
 			if (swirl->rgb_values != NULL)
 				XFree((void *) swirl->rgb_values);
+#endif /* !STANDALONE */
 			if (swirl->ximage != NULL)
 				XDestroyImage(swirl->ximage);
 			if (swirl->knots)
@@ -1471,8 +1467,8 @@ release_swirl(ModeInfo * mi)
 
 /****************************************************************/
 
-void
-refresh_swirl(ModeInfo * mi)
+ENTRYPOINT void
+refresh_swirl (ModeInfo * mi)
 {
 	SWIRL_P     swirl = &(swirls[MI_SCREEN(mi)]);
 
@@ -1482,3 +1478,5 @@ refresh_swirl(ModeInfo * mi)
 		swirl->drawing = False;
 	}
 }
+
+XSCREENSAVER_MODULE ("Swirl", swirl)
